# This test is a simple example script to show the usage of various objects 
# like FTP HTTP clients and servers.


set stopTime 300.0
set segperack 2
set segsize 1460
set delack 0.5
set startTime 1.0
Agent/TCP set window_ 15
set win_size 1
Agent/TCP set windowInit_ $win_size
set num_webs 4
set buffersize 25
set lastsample 0
set client_addr 0
set no_of_inlines 3
set filesize 1000000


# builds a 10 node topology with 4 clients and 4 servers connected by two
# intermediate routers. The links between the clients and the intermediate
# router and the servers and intermediate routers are 10Mbps while the link
# between the two intermediate routers is the bottlenect link with a BW of 
# 1.5Mbps


proc build_topology { ns which } {
	global buffersize 
	foreach i "0 1 2 3 4 5 6 7 8 9" {
		global n$i
		set n$i [$ns node]
	    [set n${i}] shape circle
	    [set n${i}] color green
	}

     	if { $which == "FIFO" } {
		# congested link
		$ns duplex-link $n0 $n1 1.5Mb 20ms DropTail
		$ns queue-limit $n0 $n1 $buffersize
		$ns queue-limit $n1 $n0 $buffersize
		# clients
		foreach j "2 3 4 5" {
		        set dly [expr ($j-1)]
		        $ns duplex-link [set n$j] $n0 10.0Mb ${dly}ms DropTail
			$ns queue-limit [set n$j] $n0 $buffersize
			$ns queue-limit $n0 [set n$j] $buffersize
		}
		# servers
		$ns duplex-link $n1 $n6 10.0Mb  100us DropTail
		$ns duplex-link $n1 $n7 10.0Mb  10ms   DropTail
		$ns duplex-link $n1 $n8 10.0Mb  30ms  DropTail
		$ns duplex-link $n1 $n9 10.0Mb  60ms  DropTail
		foreach j "6 7 8 9" {
		$ns queue-limit $n1 [set n$j] $buffersize
		$ns queue-limit [set n$j] $n1 $buffersize
		}
	} elseif { $which == "RED" } {
		# congested link
	       
	        $ns duplex-link $n0 $n1 1.5Mb 20ms RED
		$ns queue-limit $n0 $n1 $buffersize
		$ns queue-limit $n1 $n0 $buffersize

	# clients
		foreach j "2 3 4 5" {
		        set dly [expr ($j-1)]
			$ns duplex-link [set n$j] $n0 10.0Mb ${dly}ms DropTail
			$ns queue-limit [set n$j] $n0 $buffersize
			$ns queue-limit $n0 [set n$j] $buffersize
		}
		# servers
		$ns duplex-link $n1 $n6 10.0Mb  100us DropTail
		$ns duplex-link $n1 $n7 10.0Mb  10ms   DropTail
		$ns duplex-link $n1 $n8 10.0Mb  30ms  DropTail
		$ns duplex-link $n1 $n9 10.0Mb  60ms  DropTail
		foreach j "6 7 8 9" {
		$ns queue-limit $n1 [set n$j] $buffersize
		$ns queue-limit [set n$j] $n1 $buffersize
		}
	} else {
		# congested link
		$ns duplex-link $n0 $n1 1.5Mb 50ms FQ
		$ns queue-limit $n0 $n1 $buffersize
		$ns queue-limit $n1 $n0 $buffersize
		# clients
		foreach j "2 3 4 5" {
			$ns duplex-link [set n$j] $n0 10.0Mb 100us FQ
			$ns queue-limit [set n$j] $n0 $buffersize
			$ns queue-limit $n0 [set n$j] $buffersize
		}
		# servers
		foreach j "6 7 8 9" {
		$ns duplex-link $n1 [set n$j] 10.0Mb 100us FQ
		$ns queue-limit $n1 [set n$j] $buffersize
		$ns queue-limit [set n$j] $n1 $buffersize
		}
	}
}


# routines to build a FTP client.
# sets various paramters like the Flow_id apart from getting a ftp client
# object.


proc build_ftpclient {cnd snd sftp startTime Flow_id} {
    
    global ns
    global stopTime
    set cli [get_ftpclient]
    set ctcp [get_fulltcp]
    $ctcp attach-application $cli
     $ctcp set fid_ $Flow_id
    $cli tcp $ctcp
    $ns attach-agent $cnd $ctcp

    set stcp [get_fulltcp]
    $stcp attach-application $sftp
     $stcp set fid_ $Flow_id
    $ns attach-agent $snd $stcp

    $ns connect $ctcp $stcp
    $ctcp set dst_ [$stcp set addr_]
    $stcp listen
    $ns at $startTime "$cli start"
    $ns at $stopTime "$cli stop"
    global ftplist
    global ftplist
    set ftplist($ctcp) $stcp
    return $cli
}

# routines to build a FTP client.
# sets various paramters like the Flow_id apart from getting a web client
# object. Here one has to obtain tcp agents to establish the in-line URL
# connections apart from the primary URL connection.



proc build_wwwclient {cnd snd sp si startTime Flow_id} {

    global ns
    global stopTime
    global no_of_inlines

    set cli [get_wwwclient]
    set ctcp [get_fulltcp]
    $ctcp attach-application $cli
    $ctcp set fid_ $Flow_id
    $cli tcp-primary $ctcp
    $ns attach-agent $cnd $ctcp
    
    set stcp [get_fulltcp]
    $stcp attach-application $sp
    $stcp set fid_ $Flow_id
    $ns attach-agent $snd $stcp

    $ns connect $ctcp $stcp
    $ctcp set dst_ [$stcp set addr_]
    $stcp listen
    global wwwlist
    set wwwlist($ctcp) $stcp
    
    for { set i 1 } {$i <= $no_of_inlines } { incr i 1 } {
	
	set ctcp [get_fulltcp]
	$ctcp attach-application $cli
	$ctcp set fid_ $Flow_id
	$cli tcp-in-line $ctcp
	$ns attach-agent $cnd $ctcp
	
	set stcp [get_fulltcp]
	$stcp set fid_ $Flow_id
	$stcp attach-application $si
	
	$ns attach-agent $snd $stcp
    
	$ns connect $ctcp $stcp
	$ctcp set dst_ [$stcp set addr_]
	$stcp listen
	set wwwlist($ctcp) $stcp
    }

    $ns at $startTime "$cli start"
    $ns at $stopTime "$cli stop"
    
    return $cli
}

# called from within build_ftpclient
proc get_ftpclient {} {

    global client_addr
    set cli [new Agent/TcpApp/FtpClient]
    $cli set addr_ [incr client_addr]
    return $cli
}

# called from within build_wwwclient
proc get_wwwclient {} {
	global client_addr	
	set cli [new Agent/TcpApp/WWWClient]
	$cli set addr_ [incr client_addr]
	$cli max_think_time 1.0
	return $cli
	}

proc get_fulltcp {} {

    global segperack
    global segsize
    global delack
    set atcp [new Agent/TCP/FullTcp]
    $atcp set segsperack_ $segperack
    $atcp set segsize_ $segsize
    $atcp set interval_ $delack
    return $atcp
}
    


proc uniform {a b} {
	expr $a + (($b- $a) * ([ns-random]*1.0/0x7fffffff))
}

proc finish {} {

    global ns f01 f10
    $ns flush-trace
    close $f01
    close $f10
    exit 0
}




proc traceQueues { node traceFile } {
        global ns
	foreach nbr [$node neighbors] {
		$ns namtrace-queue $node $nbr $traceFile
		[$ns link $node $nbr] trace-dynamics $ns $traceFile
	}
}


set f01 [open out.tr01 w]
set f10 [open out.tr10 w]
build_topology $ns FIFO


##########################################
# setting up FTP servers and building the corresponding clients.
###################################


for {set k 1} {$k <= 1} {incr k 1} {
    for {set j 2} {$j <= 2} {incr j 1} {
	set i [expr $j+4]
	set sftp [new Agent/TcpApp/FtpServer]
	$sftp file_size $filesize
	build_ftpclient [set n$j] [set n$i]  $sftp [uniform 0.0 1.0]
    }
}

#########################
# setting up WWW servers and building the corresponding clients.
#####################

for {set k 1} {$k <= $num_webs} {incr k 1} {
    for {set i 1} {$i <= 2} {incr i 1} {
	for {set j 2} {$j <= 5} {incr j 1} { 
	    set siwww [new Agent/TcpApp/WWWServer]
	    set spwww [new Agent/TcpApp/WWWServer]
	    $spwww primary
	    build_wwwclient [set n$j] [set n[expr 4+$j]] $spwww $siwww [uniform 0.0 1.0] [expr $j-1]
	}
    }
}


#####################################


# setting up the necessary trace files.

$ns namtrace-queue $n42 $n43 $f01
$ns namtrace-queue $n43 $n42 $f10

$ns at [expr $stopTime + 30.0] "finish"
$ns run
exit 0



