/*
 * Copyright (c) 1991-1997 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /usr/src/mash/repository/vint/ns-2/tcp.h,v 1.22 1997/07/25 06:54:36 sfloyd Exp $ (LBL)
 */
#ifndef ns_www_h
#define ns_www_h

#include "packet.h"
#include "agent.h"

/* states */

#define THINKING 0
#define REQ_SENT 1
#define INLINES  2

/* events */

#define NEW_PAGE 0
#define TIMEOUT  1
#define MAX_INLINES 8

class WWWClientAgent;

class NewPageTimer : public TimerHandler {
public:
  NewPageTimer(WWWClientAgent *a) : TimerHandler() {a_ = a;}
protected:
  virtual void expire(Event *e);
  WWWClientAgent *a_;
};


class TimeOutTimer : public TimerHandler {
public:
  TimeOutTimer(WWWClientAgent *a) : TimerHandler() {a_ = a;}
protected:
  virtual void expire(Event *e);
  WWWClientAgent *a_;
};


class WWWClientAgent : public TcpAppAgent {
 public:
	WWWClientAgent();
	int command(int argc, const char*const* argv);
	void recv(Packet*, FullTcpAgent*);
	virtual void timeout(int);
 protected:

	void start();
	void stop();
	double now()  { Scheduler::instance().clock(); }
	int send_primary();
	int send_in_line(int);
	int running_;
	double max_think_time_;
	double start_trans_;
	int state_;
	int in_line_docs_;
	int pending_docs_;
	int url_;
	FullTcpAgent* primary_;
	FullTcpAgent* in_line_[MAX_INLINES];
	static FILE* fp_;
	NewPageTimer newpage_timer_;
	TimeOutTimer timeout_timer_;
};





#endif
