
/*
 * For use with tcp-full
 * The cliet browser uses the target agent as the tcp connection to
 * the server agent that sends primary docs. There's an array of
 * in_line_ that points to the tcp connections to the server agent that
 * sends in-line docs. tcp-full's advance() method is used to pass the url
 * size and to indicate not to close the connection when it is sent.
 * The url size could be randomized, but is fixed here. This agent doesn't
 * care what size the data file was. It would be possible to use the recv()
 * method and pass this agent's address as the handler, but that requires
 * some architectural rethink of the existing ns-2 and that is not my concern.
 * This will only work with a very slightly modified version of tcp-full.
 * Here the recv() function is called when the transaction
 * is done, but doesn't pass a packet.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: wwwc.cc 97/07/31 Exp $ (LBL)";
#endif

#include "tcp-full.h"
#include "Tcl.h"
#include "random.h"
#include "trace.h"
#include "www.h"

FILE* WWWClientAgent::fp_ = fopen("pagedly.tr", "w");

static class WWWClientClass : public TclClass {
public:
	WWWClientClass() : TclClass("Agent/TcpApp/WWWClient") {}
	TclObject* create(int, const char*const*) {
		return (new WWWClientAgent());
	}
} class_wwwc;

WWWClientAgent::WWWClientAgent() : TcpAppAgent(PT_NTYPE), in_line_docs_(0), running_(0), newpage_timer_(this), timeout_timer_(this)
{
	max_think_time_ = 10.0;
//	bind_time("max_think_time_", &max_think_time_);
}

void WWWClientAgent::start()
{
	running_ = 1;
	url_ = 210;
	state_ = THINKING;
	newpage_timer_.resched(0.);
}

void WWWClientAgent::stop()
{
	running_ = 0;
}

void WWWClientAgent::timeout(int event_type)
{
	if (running_)
		if(event_type == NEW_PAGE)	{
			if(send_primary())	{
				start_trans_ = now();
				state_ = REQ_SENT;
				timeout_timer_.resched(10.0);
			} else {
				printf("wwwc %d failed to start at %f\n", addr_, now());
				double t = Random::uniform(1.0, max_think_time_);
				newpage_timer_.resched(t);
			}
		} else if(event_type == TIMEOUT)	{
			printf("wwwc %d timed out at %f\n", addr_, now());
		}
}

int WWWClientAgent::send_primary()
{
	if(primary_->state() != 0)
		return 0;
	for(int i=0; i<in_line_docs_; i++)
		if(in_line_[i]->state() != 0)
			return 0;
	//this calls special advance that takes bytes,
	//	0 flag says don't close the connection
	return primary_->advance(url_, 0);	
}

int WWWClientAgent::send_in_line(int i)
{
	//this calls special advance that takes bytes,
	//	0 flag says don't close the connection
	return in_line_[i]->advance(url_, 0);
}

//scheduled only when the tcp connection(s) are upcalling
// assuming that the server sends a FIN and the connection closes itself
void WWWClientAgent::recv(Packet*, FullTcpAgent* tcp)
{
	if(state_ == INLINES && tcp == primary_)
		printf("wwwc: error in agent %d at %g\n", addr_, now());
	if(state_ == REQ_SENT)	{
		for(int i=0; i<in_line_docs_; i++)
			if(!send_in_line(i))
				printf("wwwc %d failed to send in-line at %f\n", addr_, now());
		if((pending_docs_ = in_line_docs_) != 0)
			state_ = INLINES;
	}
	else if(state_ == INLINES)	{
		--pending_docs_;
	}
	if(state_ != THINKING && pending_docs_ == 0)	{
		fprintf(fp_, "%g %d %g\n", now(), addr_, now() - start_trans_);
		//double t = exp(1.5)*Random::weibull(0.4);
		double t = Random::uniform(1.0, max_think_time_);
		/*cancel(TIMEOUT); */
		newpage_timer_.resched(t);
		state_ = THINKING;
	}
	else if(state_ == THINKING)
		printf("wwwc %d recv got called at %f\n", addr_, now());
}

/*
 * used to set the pointers to the tcp agents to send urls to and
 *	to set the number of in-lines
 */
int WWWClientAgent::command(int argc, const char*const* argv)
{
	Tcl& tcl = Tcl::instance();
	if (argc == 2) {
		if (strcmp(argv[1], "start") == 0) {
			start();
			return(TCL_OK);
		} else if (strcmp(argv[1], "stop") == 0) {
			stop();
			return(TCL_OK);
		}
	}
	if (argc == 3) {
		if (strcmp(argv[1], "tcp-primary") == 0) { 
			primary_ = (FullTcpAgent *)TclObject::lookup(argv[2] ); 
			if (primary_ == 0) {
				tcl.resultf("no such agent %s", argv[2]);
				return(TCL_ERROR);
			}
			return(TCL_OK);
		}
		if (strcmp(argv[1], "tcp-in-line") == 0) { 
			if(in_line_docs_ == MAX_INLINES)
				tcl.resultf("too many in-lines");
			in_line_[in_line_docs_] = (FullTcpAgent *)TclObject::lookup(argv[2] ); 
			if (in_line_[in_line_docs_++] == 0) {
				tcl.resultf("no such agent %s", argv[2]);
				return(TCL_ERROR);
			}
			return(TCL_OK);
		}
		if (strcmp(argv[1], "max_think_time") == 0) { 
			max_think_time_ = atof(argv[2]); 
			return(TCL_OK);
		}
	}

	return (Agent::command(argc, argv));
}

void NewPageTimer::expire(Event *e) {
	a_->timeout(NEW_PAGE);
}

void TimeOutTimer::expire(Event *e) {
	a_->timeout(TIMEOUT);
}
