/*
 * This agent is paired with one or more www clients. It needs to be
 * set to return primary urls, otherwise it defaults to in-lines.
 * It expects to be the target of a FullTcpAgent.
 */
#ifndef lint
static char rcsid[] =
    "@(#) $Header: wwws.cc, Exp $ (LBL)";
#endif

#include "tcp-full.h"
#include "Tcl.h"
#include "random.h"
#include "trace.h"

class WWWSrvrAgent : public TcpAppAgent {
 public:
	WWWSrvrAgent();
	int command(int argc, const char*const* argv);
	void recv(Packet*, FullTcpAgent*);
protected:
	double now()  { Scheduler::instance().clock(); }
	int primary_;		//set if this services primary urls
	int min_response_;	//number of bytes in min response file
	int max_response_;	//number of bytes in max response file
	double shape_;		//the pareto distribution shape parameter
	static FILE* fp_;
};

FILE* WWWSrvrAgent::fp_ = fopen("filesize.tr", "w");

static class WWWSrvrClass : public TclClass {
public:
	WWWSrvrClass() : TclClass("Agent/TcpApp/WWWServer") {}
	TclObject* create(int, const char*const*) {
		return (new WWWSrvrAgent());
	}
} class_wwws;

WWWSrvrAgent::WWWSrvrAgent() : TcpAppAgent(PT_NTYPE), primary_(0)
{
//	bind("min_byte_response", &min_response_);
//	bind("max_byte_response", &max_response_);
}

//should only be called when a full URL is received from a client
//need to make sure it goes to the right tcp connection

void WWWSrvrAgent::recv(Packet*, FullTcpAgent* tcp)
{
	int length;
	if(primary_)	{
	  if(Random::uniform() < 0.1)
			length = 500;
		else {
			length = (int)Random::pareto(1000., 0.91);
			if(length > 15000)
				length = 15000; //in Bytes
		}
	}
	else	{
		if(Random::uniform() < 0.1)
			length = 500;
		else {
			length = (int)Random::pareto(1000., 1.24);
			if(length > 25000)
				length = 25000; //in Bytes
		}
	}
	tcp->advance(length, 1);
	fprintf(fp_, "%g %d %d\n", now(), primary_, length);
}

/*
 * set to primary
 */
int WWWSrvrAgent::command(int argc, const char*const* argv)
{
	if (argc == 2) {
		if (strcmp(argv[1], "primary") == 0) {
			primary_ = 1;
			return (TCL_OK); 
		}
	}
	return (Agent::command(argc, argv));
}

